@@ -5,7 +5,6 @@ import sys
 from enum import Enum
 from gettext import gettext as _
 from typing import (
-    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -23,47 +22,37 @@ import click
 import click.core
 import click.formatting
 import click.parser
+import click.shell_completion
 import click.types
 import click.utils
-from typer.completion import completion_init
-
-from ._compat_utils import _get_click_major
 
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
     from typing_extensions import Literal
 
+MarkupMode = Literal["markdown", "rich", None]
+
 try:
     import rich
 
     from . import rich_utils
 
-except ImportError:  # pragma: nocover
+    DEFAULT_MARKUP_MODE: MarkupMode = "rich"
+
+except ImportError:  # pragma: no cover
     rich = None  # type: ignore
-
-if TYPE_CHECKING:  # pragma: no cover
-    if _get_click_major() == 7:
-        import click.shell_completion
-
-MarkupMode = Literal["markdown", "rich", None]
+    DEFAULT_MARKUP_MODE = None
 
 
-# TODO: when deprecating Click 7, remove this
-def _typer_param_shell_complete(
-    self: click.core.Parameter, ctx: click.Context, incomplete: str
-) -> List["click.shell_completion.CompletionItem"]:
-    if self._custom_shell_complete is not None:
-        results = self._custom_shell_complete(ctx, self, incomplete)
-
-        if results and isinstance(results[0], str):
-            from click.shell_completion import CompletionItem
-
-            results = [CompletionItem(c) for c in results]
-
-        return cast(List["click.shell_completion.CompletionItem"], results)
-
-    return self.type.shell_complete(ctx, self, incomplete)
+# Copy from click.parser._split_opt
+def _split_opt(opt: str) -> Tuple[str, str]:
+    first = opt[:1]
+    if first.isalnum():
+        return "", opt
+    if opt[1:2] == first:
+        return opt[:2], opt[2:]
+    return first, opt[1:]
 
 
 def _typer_param_setup_autocompletion_compat(
@@ -73,17 +62,18 @@ def _typer_param_setup_autocompletion_compat(
         Callable[[click.Context, List[str], str], List[Union[Tuple[str, str], str]]]
     ] = None,
 ) -> None:
-    if autocompletion is not None and self._custom_shell_complete is None:
+    if self._custom_shell_complete is not None:
         import warnings
 
         warnings.warn(
-            "'autocompletion' is renamed to 'shell_complete'. The old name is"
-            " deprecated and will be removed in Click 8.1. See the docs about"
-            " 'Parameter' for information about new behavior.",
+            "In Typer, only the parameter 'autocompletion' is supported. "
+            "The support for 'shell_complete' is deprecated and will be removed in upcoming versions. ",
             DeprecationWarning,
             stacklevel=2,
         )
 
+    if autocompletion is not None:
+
         def compat_autocompletion(
             ctx: click.Context, param: click.core.Parameter, incomplete: str
         ) -> List["click.shell_completion.CompletionItem"]:
@@ -91,14 +81,15 @@ def _typer_param_setup_autocompletion_compat(
 
             out = []
 
-            for c in autocompletion(ctx, [], incomplete):  # type: ignore
+            for c in autocompletion(ctx, [], incomplete):
                 if isinstance(c, tuple):
-                    c = CompletionItem(c[0], help=c[1])
-                elif isinstance(c, str):
-                    c = CompletionItem(c)
+                    use_completion = CompletionItem(c[0], help=c[1])
+                else:
+                    assert isinstance(c, str)
+                    use_completion = CompletionItem(c)
 
-                if c.value.startswith(incomplete):
-                    out.append(c)
+                if use_completion.value.startswith(incomplete):
+                    out.append(use_completion)
 
             return out
 
@@ -125,7 +116,7 @@ def _get_default_string(
         )
     elif isinstance(default_value, Enum):
         default_string = str(default_value.value)
-    elif callable(default_value):
+    elif inspect.isfunction(default_value):
         default_string = _("(dynamic)")
     elif isinstance(obj, TyperOption) and obj.is_bool_flag and obj.secondary_opts:
         # For boolean flags that have distinct True/False opts,
@@ -136,11 +127,11 @@ def _get_default_string(
         # )[1]
         if obj.default:
             if obj.opts:
-                default_string = click.parser.split_opt(obj.opts[0])[1]
+                default_string = _split_opt(obj.opts[0])[1]
             else:
                 default_string = str(default_value)
         else:
-            default_string = click.parser.split_opt(obj.secondary_opts[0])[1]
+            default_string = _split_opt(obj.secondary_opts[0])[1]
         # Typer override end
     elif (
         isinstance(obj, TyperOption)
@@ -166,13 +157,7 @@ def _extract_default_help_str(
     ctx.resilient_parsing = True
 
     try:
-        if _get_click_major() > 7:
-            default_value = obj.get_default(ctx, call=False)
-        else:
-            if inspect.isfunction(obj.default):
-                default_value = _("(dynamic)")
-            else:
-                default_value = obj.default
+        default_value = obj.get_default(ctx, call=False)
     finally:
         ctx.resilient_parsing = resilient
     return default_value
@@ -186,6 +171,7 @@ def _main(
     complete_var: Optional[str] = None,
     standalone_mode: bool = True,
     windows_expand_args: bool = True,
+    rich_markup_mode: MarkupMode = DEFAULT_MARKUP_MODE,
     **extra: Any,
 ) -> Any:
     # Typer override, duplicated from click.main() to handle custom rich exceptions
@@ -201,23 +187,10 @@ def _main(
         args = list(args)
 
     if prog_name is None:
-        if _get_click_major() > 7:
-            prog_name = click.utils._detect_program_name()
-        else:
-            from click.utils import make_str
-
-            prog_name = make_str(
-                os.path.basename(sys.argv[0] if sys.argv else __file__)
-            )
+        prog_name = click.utils._detect_program_name()
 
     # Process shell completion requests and exit early.
-    if _get_click_major() > 7:
-        self._main_shell_completion(extra, prog_name, complete_var)
-    else:
-        completion_init()
-        from click.core import _bashcomplete  # type: ignore
-
-        _bashcomplete(self, prog_name, complete_var)
+    self._main_shell_completion(extra, prog_name, complete_var)
 
     try:
         try:
@@ -233,14 +206,16 @@ def _main(
                 # even always obvious that `rv` indicates success/failure
                 # by its truthiness/falsiness
                 ctx.exit()
-        except (EOFError, KeyboardInterrupt):
+        except EOFError as e:
             click.echo(file=sys.stderr)
-            raise click.Abort()
+            raise click.Abort() from e
+        except KeyboardInterrupt as e:
+            raise click.exceptions.Exit(130) from e
         except click.ClickException as e:
             if not standalone_mode:
                 raise
             # Typer override
-            if rich:
+            if rich and rich_markup_mode is not None:
                 rich_utils.rich_format_error(e)
             else:
                 e.show()
@@ -270,7 +245,7 @@ def _main(
         if not standalone_mode:
             raise
         # Typer override
-        if rich:
+        if rich and rich_markup_mode is not None:
             rich_utils.rich_abort_error()
         else:
             click.echo(_("Aborted!"), file=sys.stderr)
@@ -293,6 +268,8 @@ class TyperArgument(click.core.Argument):
         expose_value: bool = True,
         is_eager: bool = False,
         envvar: Optional[Union[str, List[str]]] = None,
+        # Note that shell_complete is not fully supported and will be removed in future versions
+        # TODO: Remove shell_complete in a future version (after 0.16.0)
         shell_complete: Optional[
             Callable[
                 [click.Context, click.Parameter, str],
@@ -315,27 +292,21 @@ class TyperArgument(click.core.Argument):
         self.show_envvar = show_envvar
         self.hidden = hidden
         self.rich_help_panel = rich_help_panel
-        kwargs: Dict[str, Any] = {
-            "param_decls": param_decls,
-            "type": type,
-            "required": required,
-            "default": default,
-            "callback": callback,
-            "nargs": nargs,
-            "metavar": metavar,
-            "expose_value": expose_value,
-            "is_eager": is_eager,
-            "envvar": envvar,
-        }
-        if _get_click_major() > 7:
-            kwargs["shell_complete"] = shell_complete
-        else:
-            kwargs["autocompletion"] = autocompletion
-        super().__init__(**kwargs)
-        if _get_click_major() > 7:
-            _typer_param_setup_autocompletion_compat(
-                self, autocompletion=autocompletion
-            )
+
+        super().__init__(
+            param_decls=param_decls,
+            type=type,
+            required=required,
+            default=default,
+            callback=callback,
+            nargs=nargs,
+            metavar=metavar,
+            expose_value=expose_value,
+            is_eager=is_eager,
+            envvar=envvar,
+            shell_complete=shell_complete,
+        )
+        _typer_param_setup_autocompletion_compat(self, autocompletion=autocompletion)
 
     def _get_default_string(
         self,
@@ -398,8 +369,13 @@ class TyperArgument(click.core.Argument):
         if self.required:
             extra.append(_("required"))
         if extra:
-            extra_str = ";".join(extra)
-            help = f"{help}  [{extra_str}]" if help else f"[{extra_str}]"
+            extra_str = "; ".join(extra)
+            extra_str = f"[{extra_str}]"
+            if rich is not None:
+                # This is needed for when we want to export to HTML
+                extra_str = rich.markup.escape(extra_str).strip()
+
+            help = f"{help}  {extra_str}" if help else f"{extra_str}"
         return name, help
 
     def make_metavar(self) -> str:
@@ -409,7 +385,7 @@ class TyperArgument(click.core.Argument):
             return self.metavar
         var = (self.name or "").upper()
         if not self.required:
-            var = "[{}]".format(var)
+            var = f"[{var}]"
         type_var = self.type.get_metavar(self)
         if type_var:
             var += f":{type_var}"
@@ -417,11 +393,6 @@ class TyperArgument(click.core.Argument):
             var += "..."
         return var
 
-    def shell_complete(
-        self, ctx: click.Context, incomplete: str
-    ) -> List["click.shell_completion.CompletionItem"]:
-        return _typer_param_shell_complete(self, ctx=ctx, incomplete=incomplete)
-
 
 class TyperOption(click.core.Option):
     def __init__(
@@ -438,6 +409,8 @@ class TyperOption(click.core.Option):
         expose_value: bool = True,
         is_eager: bool = False,
         envvar: Optional[Union[str, List[str]]] = None,
+        # Note that shell_complete is not fully supported and will be removed in future versions
+        # TODO: Remove shell_complete in a future version (after 0.16.0)
         shell_complete: Optional[
             Callable[
                 [click.Context, click.Parameter, str],
@@ -452,7 +425,6 @@ class TyperOption(click.core.Option):
         prompt_required: bool = True,
         hide_input: bool = False,
         is_flag: Optional[bool] = None,
-        flag_value: Optional[Any] = None,
         multiple: bool = False,
         count: bool = False,
         allow_from_autoenv: bool = True,
@@ -463,43 +435,33 @@ class TyperOption(click.core.Option):
         # Rich settings
         rich_help_panel: Union[str, None] = None,
     ):
-        # TODO: when deprecating Click 7, remove custom kwargs with prompt_required
-        # and call super().__init__() directly
-        kwargs: Dict[str, Any] = {
-            "param_decls": param_decls,
-            "type": type,
-            "required": required,
-            "default": default,
-            "callback": callback,
-            "nargs": nargs,
-            "metavar": metavar,
-            "expose_value": expose_value,
-            "is_eager": is_eager,
-            "envvar": envvar,
-            "show_default": show_default,
-            "prompt": prompt,
-            "confirmation_prompt": confirmation_prompt,
-            "hide_input": hide_input,
-            "is_flag": is_flag,
-            "flag_value": flag_value,
-            "multiple": multiple,
-            "count": count,
-            "allow_from_autoenv": allow_from_autoenv,
-            "help": help,
-            "hidden": hidden,
-            "show_choices": show_choices,
-            "show_envvar": show_envvar,
-        }
-        if _get_click_major() > 7:
-            kwargs["prompt_required"] = prompt_required
-            kwargs["shell_complete"] = shell_complete
-        else:
-            kwargs["autocompletion"] = autocompletion
-        super().__init__(**kwargs)
-        if _get_click_major() > 7:
-            _typer_param_setup_autocompletion_compat(
-                self, autocompletion=autocompletion
-            )
+        super().__init__(
+            param_decls=param_decls,
+            type=type,
+            required=required,
+            default=default,
+            callback=callback,
+            nargs=nargs,
+            metavar=metavar,
+            expose_value=expose_value,
+            is_eager=is_eager,
+            envvar=envvar,
+            show_default=show_default,
+            prompt=prompt,
+            confirmation_prompt=confirmation_prompt,
+            hide_input=hide_input,
+            is_flag=is_flag,
+            multiple=multiple,
+            count=count,
+            allow_from_autoenv=allow_from_autoenv,
+            help=help,
+            hidden=hidden,
+            show_choices=show_choices,
+            show_envvar=show_envvar,
+            prompt_required=prompt_required,
+            shell_complete=shell_complete,
+        )
+        _typer_param_setup_autocompletion_compat(self, autocompletion=autocompletion)
         self.rich_help_panel = rich_help_panel
 
     def _get_default_string(
@@ -522,9 +484,6 @@ class TyperOption(click.core.Option):
         return _extract_default_help_str(self, ctx=ctx)
 
     def get_help_record(self, ctx: click.Context) -> Optional[Tuple[str, str]]:
-        # Click 7.x was not breaking this use case, so in that case, re-use its logic
-        if _get_click_major() < 8:
-            return super().get_help_record(ctx)
         # Duplicate all of Click's logic only to modify a single line, to allow boolean
         # flags with only names for False values as it's currently supported by Typer
         # Ref: https://typer.tiangolo.com/tutorial/parameter-types/bool/#only-names-for-false
@@ -605,15 +564,15 @@ class TyperOption(click.core.Option):
 
         if extra:
             extra_str = "; ".join(extra)
-            help = f"{help}  [{extra_str}]" if help else f"[{extra_str}]"
+            extra_str = f"[{extra_str}]"
+            if rich is not None:
+                # This is needed for when we want to export to HTML
+                extra_str = rich.markup.escape(extra_str).strip()
+
+            help = f"{help}  {extra_str}" if help else f"{extra_str}"
 
         return ("; " if any_prefix_is_slash else " / ").join(rv), help
 
-    def shell_complete(
-        self, ctx: click.Context, incomplete: str
-    ) -> List["click.shell_completion.CompletionItem"]:
-        return _typer_param_shell_complete(self, ctx=ctx, incomplete=incomplete)
-
 
 def _typer_format_options(
     self: click.core.Command, *, ctx: click.Context, formatter: click.HelpFormatter
@@ -674,7 +633,7 @@ class TyperCommand(click.core.Command):
         hidden: bool = False,
         deprecated: bool = False,
         # Rich settings
-        rich_markup_mode: MarkupMode = None,
+        rich_markup_mode: MarkupMode = DEFAULT_MARKUP_MODE,
         rich_help_panel: Union[str, None] = None,
     ) -> None:
         super().__init__(
@@ -725,11 +684,12 @@ class TyperCommand(click.core.Command):
             complete_var=complete_var,
             standalone_mode=standalone_mode,
             windows_expand_args=windows_expand_args,
+            rich_markup_mode=self.rich_markup_mode,
             **extra,
         )
 
     def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
-        if not rich:
+        if not rich or self.rich_markup_mode is None:
             return super().format_help(ctx, formatter)
         return rich_utils.rich_format_help(
             obj=self,
@@ -747,7 +707,7 @@ class TyperGroup(click.core.Group):
             Union[Dict[str, click.Command], Sequence[click.Command]]
         ] = None,
         # Rich settings
-        rich_markup_mode: MarkupMode = None,
+        rich_markup_mode: MarkupMode = DEFAULT_MARKUP_MODE,
         rich_help_panel: Union[str, None] = None,
         **attrs: Any,
     ) -> None:
@@ -787,14 +747,21 @@ class TyperGroup(click.core.Group):
             complete_var=complete_var,
             standalone_mode=standalone_mode,
             windows_expand_args=windows_expand_args,
+            rich_markup_mode=self.rich_markup_mode,
             **extra,
         )
 
     def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
-        if not rich:
+        if not rich or self.rich_markup_mode is None:
             return super().format_help(ctx, formatter)
         return rich_utils.rich_format_help(
             obj=self,
             ctx=ctx,
             markup_mode=self.rich_markup_mode,
         )
+
+    def list_commands(self, ctx: click.Context) -> List[str]:
+        """Returns a list of subcommand names.
+        Note that in Click's Group class, these are sorted.
+        In Typer, we wish to maintain the original order of creation (cf Issue #933)"""
+        return [n for n, c in self.commands.items()]
